<?php

//-----------------------------------------------------------------------------------------
// function RadioBtn: Creates/returns the html code for a radio button.
// $VarName: the name of the POST or GET html variable
// $Value:   the value of "$VarName".
// $Checked: sets the radio button as checked or unchecked.
// $Break:   if it has the value "break" a html break is added as well as a \n linebreaker.  
//-----------------------------------------------------------------------------------------

function RadioBtn($VarName, $Value, $Checked, $Break){

  $CheckStr = ($Checked == "checked") ? " checked" : ""; 
  $BreakStr = ($Break == "break")     ? "<br>\n" : "";

  return "<input type='radio' name='" . $VarName . "' value='" . $Value . "'" . 
         $CheckStr . ">" . $BreakStr;
}

//-----------------------------------------------------------------------------------------
// function CheckBoxBtn: Creates/returns the html code for a checkbox.
// $VarName: the name of the POST or GET html variable
// $Value:   the value of "$VarName".
// $Checked: sets the checkbox as checked or unchecked.
// $Break:   if it has the value "break" a html break is added as well as a \n linebreaker.  
//-----------------------------------------------------------------------------------------

function CheckBoxBtn($VarName, $Value, $Checked, $Break){

  $CheckStr = ($Checked == "checked") ? " checked" : ""; 
  $BreakStr = ($Break == "break")     ? "<br>\n" : "";

  return "<input type='checkbox' name='" . $VarName . "' value='" . $Value . "'"
         . $CheckStr . ">" . $BreakStr;

}

//-----------------------------------------------------------------------------------------
// function TextField: Creates/returns the html code for a text field.
// $VarName: the name of the POST or GET html variable
// $Value:   the value of "$VarName".
// $Size:    sets the width of the text field.
// $Break:   if it has the value "break" a html break is added as well as a \n linebreaker.  
//-----------------------------------------------------------------------------------------

function TextField($VarName, $Value, $Size, $Break){

  $BreakStr = ($Break == "break")     ? "<br>\n" : "";

  return "<input type='text' name='" . $VarName . "' value='" . $Value 
         . "' size='". $Size . "'>" . $BreakStr;

}

//-----------------------------------------------------------------------------------------
// function TextArea: Creates/returns the html code for a text area.
// $VarName: the name of the POST or GET html variable
// $Value:   the value of "$VarName".
// $Size:    sets the height and width of the text area.
// $Break:   if it has the value "break" a html break is added as well as a \n linebreaker.  
//-----------------------------------------------------------------------------------------

function TextArea($VarName, $Value, $Size, $Break){

  $BreakStr = ($Break == "break")     ? "<br>\n" : "";

  return "<input type='text' name='" . $VarName . "' value='" . $Value 
         . "' size='". $Size . "'>" . $BreakStr;

}

//-----------------------------------------------------------------------------------------
// function Submit: Creates/returns the html code for a submit button.
// $Value:   the text displayed in the submit button. 
// $Break:   if it has the value "break" a html break is added as well as a \n linebreaker.  
// $Action:  if $Action is "no action" no action will take place oterwise the action value
//           is entered.
//-----------------------------------------------------------------------------------------

function Submit($Value, $Action, $Break){

  $BreakStr = ($Break == "break")     ? "<br>\n" : "";

  $ActionValue = ($Action =="no action") ? " ":" action='" . $Action . "' ";

  return "<input type='submit' value='" . $Value . "'" . $ActionValue .  $BreakStr;

}

//-----------------------------------------------------------------------------------------
// function OpacityFunc: sets the opacity according to the frequency, opacity table. 
// The returned opacity is interpolated from the two opacity values via ther corresponding 
// two frequencies between wich the user defined frequency lies.
// 
// $Frequency:   the user defined frequency in GHz.
// $FileName:    the file name of the file containg the table of frequencies and opacities.
//-----------------------------------------------------------------------------------------

function OpacityFunc($Frequency, $FileName) {

  // 'OpacityTable.dat' begin reading on row 4 (array index -> 3). Values:  80          120
  $FileData = file($FileName);

  $Opacity = "<h1>Frequency $Frequency GHz is out of range! </h1><br>";

  // Assigning frequency and Opacity values and assigning them to array elements.
  for ($k = 3; $k <= sizeof($FileData)-1; $k++){
    $row = $FileData[$k];
    $SpacePos = strpos($row, " ");
    $TableFreq = substr($row,0,$SpacePos);
    $TableOpacity = substr($row,$SpacePos,$SpacePos+10);
   
    if (is_numeric(trim($TableFreq)) and is_numeric(trim($TableOpacity))) { 
      $FrequencyArr[] = $TableFreq;
      $OpacityArr[]      = $TableOpacity;
    } // end if
  } // end for 

  // Finding the correct frequency table interval and interpolating the 
  // Opacity value to be returned.   
  for ($k = 0; $k <= sizeof($FrequencyArr)-2; $k++) {
    $FreqLow     = $FrequencyArr[$k];
    $FreqHigh    = $FrequencyArr[$k+1];
    $OpacityLow  = $OpacityArr[$k];
    $OpacityHigh = $OpacityArr[$k+1];

    // Interpolate Opacity between lower and upper frequency values.
    if ($Frequency >= $FreqLow and $Frequency <= $FreqHigh) {
      $Fraction = ($FreqHigh - $Frequency)/($FreqHigh - $FreqLow);
      $Opacity     =  $Fraction*$OpacityLow + (1.0 - $Fraction)*$OpacityHigh;       
    } // end if   
  } // end for


  return $Opacity;

}


//-----------------------------------------------------------------------------------------
// Sets the Trec value as a constant in the first HEMT intervals and interpolates a value 
// for the SIS reciever.
//-----------------------------------------------------------------------------------------

function TrecFunc($Frequency, $FileName){

  // 'TrecTable.dat' begin reading on row 4 (array index -> 3). Values:  80          120
   $FileData = file($FileName);

   $Trec = "<h1>Frequency $Frequency GHz is out of range! </h1><br>";

   // Assigning frequency and Trec values and assigning them to array elements.
   for ($k = 3; $k <= sizeof($FileData)-1; $k++){
     $row       = $FileData[$k];
     $SpacePos  = strpos($row, " "); // Position of the first: " " (space).
     $TableFreq = substr($row,0,$SpacePos); // First value in the data row.
     $TableTrec = substr($row,$SpacePos,$SpacePos+10); // Second value in the data row. 
   
     // Checking numeric values and assigning the first and second values of a row.
     if (is_numeric(trim($TableFreq)) and is_numeric(trim($TableTrec))) { 
       $FrequencyArr[] = $TableFreq;
       $TrecArr[]      = $TableTrec;
     } // end if
   } // end for 

   // Finding the correct frequency table interval and interpolating the 
   // Trec value to be returned.   
   for ($k = 0; $k <= sizeof($FrequencyArr)-2; $k++) {
  
     // Temporary "simplifying" variables.
     $FreqLow  = $FrequencyArr[$k]; 
     $FreqHigh = $FrequencyArr[$k+1];
     $TrecLow  = $TrecArr[$k];
     $TrecHigh = $TrecArr[$k+1];

     // Interpolate Trec between lower and upper frequency values.
     if ($Frequency >= $FreqLow and $Frequency <= $FreqHigh) {
       $Fraction = ($FreqHigh - $Frequency)/($FreqHigh - $FreqLow);
       $Trec     =  $Fraction*$TrecLow + (1.0 - $Fraction)*$TrecHigh;       
     } // end if   
   } // end for

  // Returning the interpolated reciever temperature.
  return $Trec;

}

//-----------------------------------------------------------------------------------------
// Calculates the Tsys value
// 
//-----------------------------------------------------------------------------------------

function TsysFunc($Trec, $Tgp, $Tatm, $Opacity, $Elevation){

  $exparg = $Opacity/sin(deg2rad($Elevation));
  $Tsys   = ($Trec + $Tgp + $Tatm*(1.0 - exp(-$exparg)))*exp($exparg); 

  return $Tsys;

}

//-----------------------------------------------------------------------------------------
// Returns the observation time in seconds.
//-----------------------------------------------------------------------------------------

function ObsTimeFunc($Tsys, $ObsModeFactor, $Sensitivity, $Resolution){

  $factor1 = ($Tsys*$ObsModeFactor)/$Sensitivity;
  return ($factor1*$factor1*pow(10,-3))/$Resolution;

}

//-----------------------------------------------------------------------------------------
// purpose: intiate post variables with default values. Returns the value of the POST 
// variable if it is already set. Othervise it returns the entered default value ($Default).
//-----------------------------------------------------------------------------------------

function InitPostVariables($PostVariable, $Default) {

  if (!isset($_POST[$PostVariable])) { 
    return $Default; }
  else {
    return $_POST[$PostVariable];
  } // end if
} // end function

//-----------------------------------------------------------------------------------------
// purpose: Creates a simple table given header information and cell in form of strings.
//-----------------------------------------------------------------------------------------

function Table($Header, $TableData, $Action) {

  $NRows = sizeof($TableData);
  $NCols = sizeof($TableData[0]);

  if (strpos($Header,"table") > 0) {
    $TableArr[] = $Header . "\n";
  }

  // Printing all cells row by row.
  for ($m = 0; $m <= $NRows-1; $m++) {
    $TableArr[] = "<tr>";
    for ($n = 0; $n <= $NCols-1; $n++) {
      if ($TableData[$m][$n] == ""){
        $TableArr[] = $TableData[$m][$n];
      }
      else if (strstr($TableData[$m][$n], 'title=')){
        $TableArr[] = "<td align='center' height='60' colspan=" . $NCols . "><h3>". 
        substr($TableData[$m][$n],6) . "</h3></td>";
      }
      else {
        $TableArr[] = "<td align='center'>" . $TableData[$m][$n]. "</td>";
        } // end if
    } // end for (inner)
    $TableArr[] = "</tr>\n";
  } // end for (outer)

  if ($Action == "end table") {
    $TableArr[] = "<br>\n </table><br>\n";
  }

  return $TableArr;

}

//-----------------------------------------------------------------------------------------
// purpose: echoes the input array.
//-----------------------------------------------------------------------------------------

  function EchoArray($StrArray) {

    for ($k = 0; $k <= sizeof($StrArray)-1; $k++){
      echo $StrArray[$k];
    } // end for

  } // end function

//-----------------------------------------------------------------------------------------
// Starting main script.
//-----------------------------------------------------------------------------------------

  // Values for buttons.
  $Checked   = "checked";
  $UnChecked = "unchecked";
  $Break     = "break";
  $NoBreak   = "nobreak";

  // Setting POST variables to their default values if the script is loaded the first time.
  $Frequency   = InitPostVariables('Frequency', 86);
  $Elevation   = InitPostVariables('Elevation', 50);
  $ObsMode     = InitPostVariables('ObsMode', 'psw');
  $Sensitivity = InitPostVariables('Sensitivity', 0.1);
  $Resolution  = InitPostVariables('Resolution', 12.5);

  // If $Obsmode is 'psw' or 'bsw' the observe mode factor is 2.0 otherwise (for fsw) 1.0.
  $ObsModeFactor = ($ObsMode == "bsw" or $ObsMode == "psw") ? 2.0:1.0; 

  // Setting and calculating the other values.
  $Tgp     = (double) 20.0; // Ground pickup temperature. Some 20K.
  $Tatm    = (double) 260.0; // Estimate of the atmosphere's temperature .


//-----------------------------------------------------------------------------------------
// Setting conditions and warning text if the user enters parameters that are out of 
// range
//-----------------------------------------------------------------------------------------

  // Making sure the user is using the right intervals for the frequency.
  $FrequencyInterval  = ($Frequency < 18 or $Frequency > 116 or
                      ($Frequency > 49.8 and $Frequency < 85)) ? "out of range":"in range";

  // If the entered frequency is out of range a red error message is displayed.
  $FrequencyText      = ($FrequencyInterval == "out of range") ? "<font color=red> 
                         Frequency out of range </font>" : "Frequency";

  // Making sure the user is using the right intervals for the frequency resolution.
  $ResolutionInterval = ($Resolution < 2 or $Resolution > 1280000) ? "out of range":"in range";

  // If the entered frequency resolution is out of range a red error message is displayed.
  $ResolutionText     = ($ResolutionInterval == "out of range") ? "<font color=red> 
                        Frequency resolution out of range </font>" : "Resolution";

  // Making sure the user is using the right intervals for the elevation.
  $ElevationInterval  = ($Elevation < 6.0 or $Elevation > 87) ? "out of range":"in range";

  // If the entered elevation is out of range a red error message is displayed.
  $ElevationText      = ($ElevationInterval == "out of range") ? "<font color=red> 
                        Elevation out of range </font>" : "Elevation";

  // Making sure the user is using a sensitivity greater than zero.
  $SensitivityInterval  = ($Sensitivity <= 0.0) ? "out of range":"in range";

  // If the entered elevation is out of range a red error message is displayed.
  $SensitivityText      = ($SensitivityInterval == "out of range") ? "<font color=red> 
                           Sensitivity not applicable </font>" : "Sensitivity";

  // Initializing error counter wich must be zero if the Tsys, Trec and the 
  // observation time, Tsys, is to be calculated and displayed.
  $n_errors = 0; 

//-----------------------------------------------------------------------------------------
//  Creating the html document
//-----------------------------------------------------------------------------------------

  echo "<html>\n";
  echo "<head>\n<title>";
  echo "Observing time estimate";
  echo "</title>\n</head>\n";
  echo "<body>\n";
  echo "<center>\n";
  echo "<table border='0' cellpadding='0' cellspacing='0' width='550'>\n";
  echo "<tr><td><h2> Observing time estimates </h2></tr></td>\n</table><br>\n";
  echo "<table border='0' cellpadding='0' cellspacing='0' width='550'>\n";
  echo "<tr><td>\n"; 

  // including explanatory text.
  include("ObserveTimeExplanation.html");
  echo "</td></tr>\n</table>\n";
  echo "\n<form action='ObservationTimeEstimate.php#BottomOfPage' method='POST'>\n";

  $TableHeader = "<table border='1' cellpadding='4' cellspacing='1' width='550'>";  

  // Creating the array of data to be used for the html table displaying buttons, data, and text.
  $TableData[] = array("<b>Property</b>", "<b>Value</b>", "<b>Unit</b>", "<b>Description</b></i>"); 

  // Radiobutton checked or unchecked status of PSW, FSW and BSW is determined.
  $PSWChecked  = ($ObsMode == "psw") ? $Checked:$UnChecked;
  $FSWChecked  = ($ObsMode == "fsw") ? $Checked:$UnChecked;
  $BSWChecked  = ($ObsMode == "bsw") ? $Checked:$UnChecked;

  // Radiobuttons for PSW, FSW and BSW.
  $TableData[] = array("PSW", RadioBtn("ObsMode", "psw", $PSWChecked, $NoBreak), 
                       "-", "<i>Position Switching</i>"); 
  $TableData[] = array("FSW", RadioBtn("ObsMode", "fsw", $FSWChecked, $NoBreak), 
                       "-", "<i>Frequency Switching</i>"); 
  $TableData[] = array("BSW", RadioBtn("ObsMode", "bsw", $BSWChecked, $NoBreak), 
                       "-", "<i>Beam Switching</i>"); 

  // Frequency text field and description.
  $TableData[] = array($FrequencyText, TextField("Frequency", $Frequency, 5, $NoBreak),
                       "[GHz]", "<i>Line frequency</i>"); 

  // Resolution text field and description.
  $TableData[] = array($ResolutionText, TextField("Resolution", $Resolution, 5, $NoBreak),
                       "[kHz]", "<i>Frequency resolution</i>"); 

  // Elevation text field and description.
  $TableData[] = array($ElevationText, TextField("Elevation", $Elevation, 5, $NoBreak), 
                       "[deg]", "<i>Typical elevation</i>"); 

  // Sensitivity text field and description.
  $TableData[] = array($SensitivityText, TextField("Sensitivity", $Sensitivity, 5, $NoBreak),
                       "[K]",   "<i>Sensitivity (r.m.s.)</i>"); 

  // submit button for calculation.
  $TableData[] = array("Submit values",Submit("Estimate!", "no action", $NoBreak), "-", 
                       "<i>Calculates the observation time</i>"); 

  // If the submit button is pressed, the php script will call itself and the second time 
  // "ObsMode" is "set" and the condition true. 
  if (isset($_POST[ObsMode])) { 

    // If frequency is out of range an error message is displayed and the error counter is
    //  incremented by one. 
    if ($FrequencyInterval == "out of range"){
      echo "<b>Frequency <font color='#FF0000'> $Frequency GHz </font> is out of range. 
           <br> Acceptable frequency intervals: <br>\n
           18 < <i>f</i> < 49.8 GHz <br> \n
           or <br> \n
           85 < <i>f</i> < 116 GHz <br><br>\n"; 
      $n_errors++;
    } // end if
    
    // If frequency is out of range an error message is displayed and the error counter is
    // incremented by one. 
    if ($ResolutionInterval == "out of range") {  
      echo "<b>Frequency resolution <font color='#FF0000'> $Resolution kHz </font> 
           is out of range.<br> Acceptable frequency resolution is within: <br> 
           2 < <i> frequency resolution </i> < 1 280 000 kHz <br><br>"; 
      $n_errors++;
    } // end if

    // If the elevation is out of range an error message is displayed and the error counter
    // is incremented by one. 
    if ($ElevationInterval == "out of range") {  
      echo "<b>Elevation <font color='#FF0000'> $Elevation degres </font> is out of range.
           <br> Acceptable elevation is within: <br>
           6 < <i> El </i> < 87 degres <br><br>"; 
      $n_errors++;
    } // end if

    // If the entered sensitivity is smaller than or equal to zero an error message is 
    // displayed and the error counter is incremented by one.
    if ($SensitivityInterval == "out of range") {  
      echo "<b>Sensitivity <font color='#FF0000'>" . $Sensitivity . " K </font> is not 
           applicable.</b> <br><br>"; 
      $n_errors++;
    } // end if 

    // if no "out of range" errors are detected, $n_errors is zero and the calculations 
    // are done and assigned to the table array. 
    if ($n_errors == 0) {  
      // Interpolating the reciever temperature from a table given the user defined 
      // frequency.
      $Trec    = (double) TrecFunc($Frequency, "TrecTable.dat"); 


      // Interpolating the opacity from a table given the user defined frequency.
      $Opacity = (double) OpacityFunc($Frequency, "OpacityTable.dat"); 

      // Calculating the system temperature Tsys.
      $Tsys    = (double) TsysFunc($Trec, $Tgp, $Tatm, $Opacity, $Elevation);

      // Calculating the observation time. 
      $ObsTime = (double) ObsTimeFunc($Tsys, $ObsModeFactor, $Sensitivity, $Resolution);

      if ($ObsTime < 1) {
        echo "<b><font color='#FF0000'> Observation time < 1 s </font></b><br><br>"; 
      } 
      else {
        // Title of the calculation part of the html table.
        $TableData[] = array("title=Calculated values","", "", ""); 

        // The reciever temperature.
        $TableData[] = array("T<sub style='vertical-align: -0.6ex;'>rec</sub>", 
                             round($Trec,0), "[K]", "<i>Reciever temperature</i>"); 

        // The system temperature.
        $TableData[] = array("T<sup style='vertical-align: 0.8ex;'>*</sup><sub style=
                           'vertical-align: -0.6ex; margin-left:-1.2ex;'>sys</sub>",
                           round($Tsys,0), "[K]", "<i>System temperature</i>"); 

        // The observation time in seconds.
        $TableData[] = array("<a name='BottomOfPage'>Observation time </a>", round($ObsTime,0), "[s]", 
                           "<i>Observation time</i>"); 
      } // end if 
    } // end if
  } // end outer if

  // All table data is saved in an array as a html table using the above defined 
  // function 'Table'.  
  $TableArr    = Table($TableHeader, $TableData, "end table");

  // The array of html code is printed to the browser using the above defined function 
  // 'TableArr'. 
  EchoArray($TableArr);

  echo "</form>\n";  
  echo "</center>\n";
  echo "</body>\n";
  echo "</html>\n";

?>
